@@ -22,10 +22,9 @@
 #include "KNSBackend.h"
 #include <KNSCore/Engine>
 #include <KShell>
+#include <KLocalizedString>
 #include <QProcess>
 #include <QRegularExpression>
-
-#include "ReviewsBackend/Rating.h"
 #include <knewstuff_version.h>
 
 KNSResource::KNSResource(const KNSCore::EntryInternal& entry, QStringList categories, KNSBackend* parent)
@@ -39,7 +38,7 @@ KNSResource::KNSResource(const KNSCore::EntryInternal& entry, QStringList catego
 
 KNSResource::~KNSResource() = default;
 
-AbstractResource::State KNSResource::state() const
+AbstractResource::State KNSResource::state()
 {
     switch(m_entry.status()) {
         case KNS3::Entry::Invalid:
@@ -226,17 +225,12 @@ void KNSResource::invokeApplication() const
     }
 }
 
-Rating * KNSResource::ratingInstance()
+QString KNSResource::executeLabel() const
 {
-    if (!m_rating) {
-        const int noc = m_entry.numberOfComments();
-        const int rating = m_entry.rating();
-        Q_ASSERT(rating <= 100);
-        return new Rating(
-            packageName(),
-            noc,
-            { { QStringLiteral("star5"), rating } }
-        );
-    }
-    return m_rating;
+    return i18n("Use");
+}
+
+QDate KNSResource::releaseDate() const
+{
+    return m_entry.updateReleaseDate().isNull() ? m_entry.releaseDate() : m_entry.updateReleaseDate();
 }
