@@ -27,7 +27,7 @@ class DiscoverNotifierDeclarativePlugin : public QQmlExtensionPlugin
     Q_OBJECT
     Q_PLUGIN_METADATA(IID "org.qt-project.Qt.QQmlExtensionInterface")
     public:
-        virtual void registerTypes(const char*) override;
+        void registerTypes(const char* /*uri*/) override;
 };
 
 #endif
